@@ -9,6 +9,8 @@
 #include <locale.h>
 #include <ctype.h>
 #include <math.h>
+#include <fstream>
+#include <stdint.h>
 
 static const char *_lf_get_lang ()
 {
@@ -90,7 +92,7 @@ LF_EXPORT lfMLstr lf_mlstr_add (lfMLstr str, const char *lang, const char *trstr
     {
         str_len = strlen (str) + 1;
         while (str [str_len])
-            str_len += 1 + strlen (str + str_len + 1);
+            str_len += 1 + strlen (str + str_len);
     }
 
     if (!lang)
@@ -460,6 +462,25 @@ float _lf_interpolate (float y1, float y2, float y3, float y4, float t)
         (t3 - t2) * tg3;
 }
 
+long int _lf_read_database_timestamp (const gchar *dirname)
+{
+    long int timestamp = -1;
+    GDir *dir = g_dir_open (dirname, 0, NULL);
+    if (dir)
+    {
+        if (g_dir_read_name (dir))
+        {
+            std::ifstream timestamp_file (g_build_filename (dirname, "timestamp.txt", NULL));
+            if (!timestamp_file.fail ())
+                timestamp_file >> timestamp;
+            else
+                timestamp = 0;
+        }
+        g_dir_close (dir);
+    }
+    return timestamp;
+}
+
 //------------------------// Fuzzy string matching //------------------------//
 
 lfFuzzyStrCmp::lfFuzzyStrCmp (const char *pattern, bool allwords)
@@ -509,10 +530,11 @@ void lfFuzzyStrCmp::Split (const char *str, GPtrArray *dest)
             while (*str && !isspace (*str) && !isdigit (*str) && !ispunct (*str))
                 str++;
 
-        // Skip solitary symbols
-        if ((str - word == 1) &&
-            (*word == '/' || *word == '-' || *word == '(' || *word == ')' ||
-             tolower (*word) == 'f'))
+        // Skip solitary symbols, including a single letter "f", except for "+"
+        // and "*", which sometimes occur in lens model names as important
+        // characters
+        if (str - word == 1 && (ispunct (*word) || tolower (*word) == 'f')
+            && *word != '*' && *word != '+')
             continue;
 
         gchar *item = g_utf8_casefold (word, str - word);
@@ -548,11 +570,11 @@ int lfFuzzyStrCmp::Compare (const char *match)
 
             if (r < 0)
             {
+                // Since our arrays are sorted, if pattern word becomes
+                // "smaller" than next word from the match, this means
+                // there's no chance anymore to find it.
                 if (match_all_words)
                 {
-                    // Since our arrays are sorted, if pattern word becomes
-                    // "smaller" than next word from the match, this means
-                    // there's no chance anymore to find it.
                     Free (match_words);
                     return 0;
                 }
@@ -568,7 +590,7 @@ int lfFuzzyStrCmp::Compare (const char *match)
         {
             if (mi >= match_words->len)
             {
-                // Found a word not present in pattern
+                // Found a word not present in match
                 Free (match_words);
                 return 0;
             }
@@ -578,14 +600,14 @@ int lfFuzzyStrCmp::Compare (const char *match)
         else
         {
             if (mi >= match_words->len)
-                // Found a word not present in pattern
+                // Found a word not present in match
                 mi = old_mi;
             else
                 mi++;
         }
     }
 
-    score = (score * 100) / pattern_words->len;
+    score = (score * 200) / (pattern_words->len + match_words->len);
 
     Free (match_words);
 
