@@ -28,54 +28,53 @@ void lfDatabase::Destroy ()
     delete static_cast<lfExtDatabase *> (this);
 }
 
+void lfExtDatabase::LoadDirectory (const gchar *dirname)
+{
+    GDir *dir = g_dir_open (dirname, 0, NULL);
+
+    if (dir)
+    {
+        GPatternSpec *ps = g_pattern_spec_new ("*.xml");
+        if (ps)
+        {
+            const gchar *fn;
+            while ((fn = g_dir_read_name (dir)))
+            {
+                size_t sl = strlen (fn);
+                if (g_pattern_match (ps, sl, fn, NULL))
+                {
+                    gchar *ffn = g_build_filename (dirname, fn, NULL);
+                    /* Ignore errors */
+                    Load (ffn);
+                    g_free (ffn);
+                }
+            }
+            g_pattern_spec_free (ps);
+        }
+        g_dir_close (dir);
+    }
+}
+
 lfError lfDatabase::Load ()
 {
-    gchar *dirs [10];
-    const gchar *const *tmp;
-    int ndirs = 0;
+    lfExtDatabase *This = static_cast<lfExtDatabase *> (this);
 
-    dirs [ndirs++] = HomeDataDir;
 #ifdef CONF_DATADIR
-    dirs [ndirs++] = (char *)CONF_DATADIR;
+    gchar *main_dirname = g_strdup (CONF_DATADIR);
+    const gchar *updates_dirname = "/var/lib/lensfun-updates";
 #else
+    /* windows based OS */
     extern gchar *_lf_get_database_dir ();
-    dirs [ndirs++] = _lf_get_database_dir ();
+    gchar *main_dirname = _lf_get_database_dir ();
+    const gchar *updates_dirname = "C:\\to\\be\\defined\\lensfun-updates";
 #endif
-    int static_ndirs = ndirs;
-    for (tmp = g_get_system_data_dirs (); ndirs < 10 && *tmp; tmp++)
-        dirs [ndirs++] = g_build_filename (*tmp, CONF_PACKAGE, NULL);
+    if (_lf_read_database_timestamp (main_dirname) > _lf_read_database_timestamp (updates_dirname))
+        This->LoadDirectory (main_dirname);
+    else
+        This->LoadDirectory (updates_dirname);
+    g_free (main_dirname);
 
-    while (ndirs > 0)
-    {
-        ndirs--;
-        GDir *dir = g_dir_open (dirs [ndirs], 0, NULL);
-
-        if (dir)
-        {
-            GPatternSpec *ps = g_pattern_spec_new ("*.xml");
-            if (ps)
-            {
-                const gchar *fn;
-                while ((fn = g_dir_read_name (dir)))
-                {
-                    size_t sl = strlen (fn);
-                    if (g_pattern_match (ps, sl, fn, NULL))
-                    {
-                        gchar *ffn = g_build_filename (dirs [ndirs], fn, NULL);
-                        /* Ignore errors */
-                        Load (ffn);
-                        g_free (ffn);
-                    }
-                }
-                g_pattern_spec_free (ps);
-            }
-            g_dir_close (dir);
-        }
-
-        /* Free all paths except the first one which is held in db struct */
-        if (ndirs >= static_ndirs)
-            g_free (dirs [ndirs]);
-    }
+    This->LoadDirectory (HomeDataDir);
 
     return LF_NO_ERROR;
 }
@@ -107,8 +106,21 @@ typedef struct
     gchar *lang;
     const gchar *stack [16];
     size_t stack_depth;
+    const char *errcontext;
 } lfParserData;
 
+static bool __chk_no_attrs(const gchar *element_name, const gchar **attribute_names,
+                           GError **error)
+{
+    if (attribute_names [0])
+    {
+        g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_UNKNOWN_ATTRIBUTE,
+                     "The <%s> element cannot have any attributes!\n", element_name);
+        return false;
+    }
+    return true;
+}
+
 static void _xml_start_element (GMarkupParseContext *context,
                                 const gchar         *element_name,
                                 const gchar        **attribute_names,
@@ -141,12 +153,17 @@ static void _xml_start_element (GMarkupParseContext *context,
             return;
         }
 
-    chk_no_attrs:
-        if (attribute_names [0])
+        int version = 0;
+        for (i = 0; attribute_names [i]; i++)
+            if (!strcmp (attribute_names [i], "version"))
+                version = atoi (attribute_values [i]);
+            else
+                goto bad_attr;
+        if (version > LF_MAX_DATABASE_VERSION)
         {
-            g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_UNKNOWN_ATTRIBUTE,
-                         "The <%s> element cannot have any attributes!\n",
-                         element_name);
+            g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
+                         "Database version is %d, but supported is only %d!\n",
+                         version, LF_MAX_DATABASE_VERSION);
             return;
         }
     }
@@ -155,12 +172,14 @@ static void _xml_start_element (GMarkupParseContext *context,
         if (ctx && !strcmp (ctx, "lensdatabase"))
         {
             pd->mount = new lfMount ();
-            goto chk_no_attrs;
+            if (!__chk_no_attrs(element_name, attribute_names, error)) return;
         }
         else if (ctx &&
                  (!strcmp (ctx, "camera") ||
                   !strcmp (ctx, "lens")))
-            goto chk_no_attrs;
+        {
+            if (!__chk_no_attrs(element_name, attribute_names, error)) return;
+        }
         else
             goto bad_ctx;
     }
@@ -169,15 +188,19 @@ static void _xml_start_element (GMarkupParseContext *context,
         if (!ctx || strcmp (ctx, "lensdatabase"))
             goto bad_ctx;
         pd->camera = new lfCamera ();
-        goto chk_no_attrs;
+        if (!__chk_no_attrs(element_name, attribute_names, error)) return;
     }
     else if (!strcmp (element_name, "lens"))
     {
         if (!ctx || strcmp (ctx, "lensdatabase"))
             goto bad_ctx;
         pd->lens = new lfLens ();
+        // Set default values for lens database entries.  This way, they are
+        // still 0 (aka "unknown") for dummy lfLens instances created ad hoc
+        // for search matching.
         pd->lens->Type = LF_RECTILINEAR;
-        goto chk_no_attrs;
+        pd->lens->AspectRatio = 1.5;
+        if (!__chk_no_attrs(element_name, attribute_names, error)) return;
     }
     else if (!strcmp (element_name, "focal"))
     {
@@ -222,32 +245,17 @@ static void _xml_start_element (GMarkupParseContext *context,
             else
                 goto bad_attr;
     }
-    else if (!strcmp (element_name, "cci"))
-    {
-        if (!ctx || strcmp (ctx, "lens") || !pd->lens)
-            goto bad_ctx;
-
-        for (i = 0; attribute_names [i]; i++)
-            if (!strcmp (attribute_names [i], "red"))
-                pd->lens->RedCCI = atof (attribute_values [i]);
-            else if (!strcmp (attribute_names [i], "green"))
-                pd->lens->GreenCCI = atof (attribute_values [i]);
-            else if (!strcmp (attribute_names [i], "blue"))
-                pd->lens->BlueCCI = atof (attribute_values [i]);
-            else
-                goto bad_attr;
-    }
     else if (!strcmp (element_name, "type"))
     {
         if (!ctx || strcmp (ctx, "lens") || !pd->lens)
             goto bad_ctx;
-        goto chk_no_attrs;
+        if (!__chk_no_attrs(element_name, attribute_names, error)) return;
     }
     else if (!strcmp (element_name, "calibration"))
     {
         if (!ctx || strcmp (ctx, "lens"))
             goto bad_ctx;
-        goto chk_no_attrs;
+        if (!__chk_no_attrs(element_name, attribute_names, error)) return;
     }
     else if (!strcmp (element_name, "distortion"))
     {
@@ -265,8 +273,6 @@ static void _xml_start_element (GMarkupParseContext *context,
                     dc.Model = LF_DIST_MODEL_POLY3;
                 else if (!strcmp (attribute_values [i], "poly5"))
                     dc.Model = LF_DIST_MODEL_POLY5;
-                else if (!strcmp (attribute_values [i], "fov1"))
-                    dc.Model = LF_DIST_MODEL_FOV1;
                 else if (!strcmp (attribute_values [i], "ptlens"))
                     dc.Model = LF_DIST_MODEL_PTLENS;
                 else
@@ -281,8 +287,7 @@ static void _xml_start_element (GMarkupParseContext *context,
             else if (!strcmp (attribute_names [i], "focal"))
                 dc.Focal = atof (attribute_values [i]);
             else if (!strcmp (attribute_names [i], "a") ||
-                     !strcmp (attribute_names [i], "k1") ||
-                     !strcmp (attribute_names [i], "omega"))
+                     !strcmp (attribute_names [i], "k1"))
                 dc.Terms [0] = atof (attribute_values [i]);
             else if (!strcmp (attribute_names [i], "b") ||
                      !strcmp (attribute_names [i], "k2"))
@@ -418,6 +423,11 @@ static void _xml_start_element (GMarkupParseContext *context,
         if (!ctx || strcmp (ctx, "calibration"))
             goto bad_ctx;
 
+        gint line, col;
+        g_markup_parse_context_get_position (context, &line, &col);
+        g_warning ("[lensfun] %s:%d:%d: <field_of_view> tag is deprecated.  Use <real-focal-length> instead",
+                   pd->errcontext, line, col);
+
         lfLensCalibFov lcf;
         memset (&lcf, 0, sizeof (lcf));
         for (i = 0; attribute_names [i]; i++)
@@ -432,6 +442,25 @@ static void _xml_start_element (GMarkupParseContext *context,
 
         pd->lens->AddCalibFov (&lcf);
     }
+    else if (!strcmp (element_name, "real-focal-length"))
+    {
+        if (!ctx || strcmp (ctx, "calibration"))
+            goto bad_ctx;
+
+        lfLensCalibRealFocal lcf;
+        memset (&lcf, 0, sizeof (lcf));
+        for (i = 0; attribute_names [i]; i++)
+            if (!strcmp (attribute_names [i], "focal"))
+                lcf.Focal = atof (attribute_values [i]);
+            else if (!strcmp (attribute_names [i], "real-focal"))
+                lcf.RealFocal = atof (attribute_values [i]);
+            else
+            {
+                goto unk_attr;
+            }
+
+        pd->lens->AddCalibRealFocal (&lcf);
+    }
     /* Handle multi-language strings */
     else if (!strcmp (element_name, "maker") ||
              !strcmp (element_name, "model") ||
@@ -445,8 +474,11 @@ static void _xml_start_element (GMarkupParseContext *context,
                 goto unk_attr;
     }
     else if (!strcmp (element_name, "compat") ||
-             !strcmp (element_name, "cropfactor"))
-        goto chk_no_attrs;
+             !strcmp (element_name, "cropfactor") ||
+             !strcmp (element_name, "aspect-ratio"))
+    {
+        if (!__chk_no_attrs(element_name, attribute_names, error)) return;
+    }
     else
         g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
                      "Unknown element <%s>!\n", element_name);
@@ -592,6 +624,19 @@ static void _xml_text (GMarkupParseContext *context,
         else
             goto bad_ctx;
     }
+    else if (!strcmp (ctx, "aspect-ratio"))
+    {
+        if (pd->lens)
+        {
+            const char *colon = strpbrk (text, ":");
+            if (colon)
+                pd->lens->AspectRatio = atof (text) / atof (colon + 1);
+            else
+                pd->lens->AspectRatio = atof (text);
+        }
+        else
+            goto bad_ctx;
+    }
     else if (!strcmp (ctx, "type"))
     {
         if (pd->lens)
@@ -658,6 +703,7 @@ lfError lfDatabase::Load (const char *errcontext, const char *data, size_t data_
     lfParserData pd;
     memset (&pd, 0, sizeof (pd));
     pd.db = This;
+    pd.errcontext = errcontext;
 
     GMarkupParseContext *mpc = g_markup_parse_context_new (
         &gmp, (GMarkupParseFlags)0, &pd, NULL);
@@ -671,7 +717,7 @@ lfError lfDatabase::Load (const char *errcontext, const char *data, size_t data_
     {
         gint line, col;
         g_markup_parse_context_get_position (mpc, &line, &col);
-        g_warning ("%s:%d:%d: %s", errcontext, line, col, err->message);
+        g_warning ("[lensfun] %s:%d:%d: %s", errcontext, line, col, err->message);
     }
 
     g_markup_parse_context_free (mpc);
@@ -818,20 +864,14 @@ char *lfDatabase::Save (const lfMount *const *mounts,
                 _lf_xml_printf (output, "\t\t<center x=\"%g\" y=\"%g\" />\n",
                                 lenses [i]->CenterX, lenses [i]->CenterY);
 
-            if (lenses [i]->RedCCI != 0 ||
-                lenses [i]->GreenCCI != 5 ||
-                lenses [i]->BlueCCI != 4)
-                _lf_xml_printf (output, "\t\t<cci red=\"%g\" green=\"%g\" blue=\"%g\" />\n",
-                                lenses [i]->RedCCI,
-                                lenses [i]->GreenCCI,
-                                lenses [i]->BlueCCI);
-
             _lf_xml_printf (output, "\t\t<cropfactor>%g</cropfactor>\n",
                             lenses [i]->CropFactor);
+            _lf_xml_printf (output, "\t\t<aspect-ratio>%g</aspect-ratio>\n",
+                            lenses [i]->AspectRatio);
 
             if (lenses [i]->CalibDistortion || lenses [i]->CalibTCA ||
                 lenses [i]->CalibVignetting || lenses [i]->CalibCrop || 
-                lenses [i]->CalibFov )
+                lenses [i]->CalibFov || lenses [i]->CalibRealFocal)
                 g_string_append (output, "\t\t<calibration>\n");
 
             if (lenses [i]->CalibDistortion)
@@ -856,12 +896,6 @@ char *lfDatabase::Save (const lfMount *const *mounts,
                                 cd->Terms [0], cd->Terms [1]);
                             break;
 
-                        case LF_DIST_MODEL_FOV1:
-                            _lf_xml_printf (
-                                output, "model=\"fov1\" omega=\"%g\" />\n",
-                                cd->Terms [0]);
-                            break;
-
                         case LF_DIST_MODEL_PTLENS:
                             _lf_xml_printf (
                                 output, "model=\"ptlens\" a=\"%g\" b=\"%g\" c=\"%g\" />\n",
@@ -888,6 +922,13 @@ char *lfDatabase::Save (const lfMount *const *mounts,
                                             ctca->Terms [0], ctca->Terms [1]);
                             break;
 
+                        case LF_TCA_MODEL_POLY3:
+                            _lf_xml_printf (output, "model=\"poly3\" vr=\"%g\" vb=\"%g\" "
+                                            "cr=\"%g\" cb=\"%g\" br=\"%g\" bb=\"%g\" />\n",
+                                            ctca->Terms [0], ctca->Terms [1], ctca->Terms [2],
+                                            ctca->Terms [3], ctca->Terms [4], ctca->Terms [5]);
+                            break;
+
                         default:
                             _lf_xml_printf (output, "model=\"none\" />\n");
                             break;
@@ -960,9 +1001,23 @@ char *lfDatabase::Save (const lfMount *const *mounts,
                 }
             }
 
+            if (lenses [i]->CalibRealFocal)
+            {
+                for (j = 0; lenses [i]->CalibRealFocal [j]; j++)
+                {
+                    lfLensCalibRealFocal *lcf = lenses [i]->CalibRealFocal [j];
+
+                    if (lcf->RealFocal>0)
+                    {
+                        _lf_xml_printf (output, "\t\t\t<real-focal-length focal=\"%g\" real-focal=\"%g\" />\n",
+                            lcf->Focal, lcf->RealFocal);
+                    };
+                }
+            }
+
             if (lenses [i]->CalibDistortion || lenses [i]->CalibTCA ||
                 lenses [i]->CalibVignetting || lenses [i]->CalibCrop ||
-                lenses [i]->CalibFov )
+                lenses [i]->CalibFov || lenses [i]->CalibRealFocal)
                 g_string_append (output, "\t\t</calibration>\n");
 
             g_string_append (output, "\t</lens>\n\n");
@@ -1103,6 +1158,20 @@ static gint _lf_compare_lens_score (gconstpointer a, gconstpointer b)
     return i2->Score - i1->Score;
 }
 
+static gint _lf_compare_lens_details (gconstpointer a, gconstpointer b)
+{
+    // Actually, we not only sort by focal length, but by MinFocal, MaxFocal,
+    // MinAperature, Maker, and Model -- in this order of priorities.
+    lfLens *i1 = (lfLens *)a;
+    lfLens *i2 = (lfLens *)b;
+
+    int cmp = _lf_lens_parameters_compare (i1, i2);
+    if (cmp != 0)
+        return cmp;
+
+    return _lf_lens_name_compare (i1, i2);
+}
+
 static void _lf_add_compat_mounts (
     const lfDatabase *This, const lfLens *lens, GPtrArray *mounts, char *mount)
 {
@@ -1144,6 +1213,7 @@ const lfLens **lfDatabase::FindLenses (const lfLens *lens, int sflags) const
     g_ptr_array_add (mounts, NULL);
 
     int score;
+    const bool sort_and_uniquify = (sflags & LF_SEARCH_SORT_AND_UNIQUIFY) != 0;
     for (size_t i = 0; i < lenses->len - 1; i++)
     {
         lfLens *dblens = static_cast<lfLens *> (g_ptr_array_index (lenses, i));
@@ -1151,7 +1221,24 @@ const lfLens **lfDatabase::FindLenses (const lfLens *lens, int sflags) const
             lens, dblens, &fc, (const char **)mounts->pdata)) > 0)
         {
             dblens->Score = score;
-            _lf_ptr_array_insert_sorted (ret, dblens, _lf_compare_lens_score);
+            if (sort_and_uniquify) {
+                bool already = false;
+                for (size_t i = 0; i < ret->len; i++)
+                {
+                    const lfLens *previous_lens = static_cast<lfLens *> (g_ptr_array_index (ret, i));
+                    if (!_lf_lens_name_compare (previous_lens, dblens))
+                    {
+                        if (dblens->Score > previous_lens->Score)
+                            ret->pdata[i] = dblens;
+                        already = true;
+                        break;
+                    }
+                }
+                if (!already)
+                    _lf_ptr_array_insert_sorted (ret, dblens, _lf_compare_lens_details);
+            }
+            else
+                _lf_ptr_array_insert_sorted (ret, dblens, _lf_compare_lens_score);
         }
     }
 
