@@ -16,7 +16,7 @@
 static struct
 {
     const char *regex;
-    guchar matchidx [4];
+    guchar matchidx [3];
     bool compiled;
     regex_t rex;
 } lens_name_regex [] =
@@ -24,19 +24,19 @@ static struct
     {
         // 1:[min aperture]-[max aperture] [min focal]-[max focal]mm
         "[[:space:]]+1:([0-9.]+)(-[0-9.]+)?[[:space:]]+([0-9.]+)(-[0-9.]+)?(mm)?",
-        { 3, 4, 1, 2 },
+        { 3, 4, 1 },
         false
     },
     {
         // [min aperture]-[max aperture]/[min focal]-[max focal]
         "([0-9.]+)(-[0-9.]+)?[[:space:]]*/[[:space:]]*([0-9.]+)(-[0-9.]+)?",
-        { 3, 4, 1, 2 },
+        { 3, 4, 1 },
         false
     },
     {
         // [min focal]-[max focal]mm f/[min aperture]-[max aperture]
         "([0-9]+[0-9.]*)(-[0-9]+[0-9.]*)?(mm)?[[:space:]]+(f/?)?([0-9.]+)(-[0-9.]+)?",
-        { 1, 2, 5, 6 },
+        { 1, 2, 5 },
         false
     }
 };
@@ -58,7 +58,7 @@ static float _lf_parse_float (const char *model, const regmatch_t &match)
 
 static bool _lf_parse_lens_name (const char *model,
                                  float &minf, float &maxf,
-                                 float &mina, float &maxa)
+                                 float &mina)
 {
     if (!model)
         return false;
@@ -83,8 +83,6 @@ static bool _lf_parse_lens_name (const char *model,
             maxf = _lf_parse_float (model, matches [matchidx [1]]);
         if (matches [matchidx [2]].rm_so != -1)
             mina = _lf_parse_float (model, matches [matchidx [2]]);
-        if (matches [matchidx [3]].rm_so != -1)
-            maxa = _lf_parse_float (model, matches [matchidx [3]]);
         return true;
     }
 
@@ -107,12 +105,12 @@ static int _lf_lens_regex_refs = 0;
 
 lfLens::lfLens ()
 {
+    // Defaults for attributes are "unknown" (mostly 0).  Otherwise, ad hoc
+    // lfLens instances used for searches could not be matched against database
+    // lenses easily.  If you need defaults for database tags, set them when
+    // reading the database.
     memset (this, 0, sizeof (*this));
-    //RedCCI = 0;
-    GreenCCI = 5;
-    BlueCCI = 4;
     Type = LF_UNKNOWN;
-    CropFactor = 1.0;
     _lf_lens_regex_refs++;
 }
 
@@ -126,6 +124,7 @@ lfLens::~lfLens ()
     _lf_list_free ((void **)CalibVignetting);
     _lf_list_free ((void **)CalibCrop);
     _lf_list_free ((void **)CalibFov);
+    _lf_list_free ((void **)CalibRealFocal);
     if (!--_lf_lens_regex_refs)
         _lf_free_lens_regex ();
 }
@@ -148,10 +147,8 @@ lfLens &lfLens::operator = (const lfLens &other)
 
     CenterX = other.CenterX;
     CenterY = other.CenterY;
-    RedCCI = other.RedCCI;
-    GreenCCI = other.GreenCCI;
-    BlueCCI = other.BlueCCI;
     CropFactor = other.CropFactor;
+    AspectRatio = other.AspectRatio;
     Type = other.Type;
 
     lf_free (CalibDistortion); CalibDistortion = NULL;
@@ -174,6 +171,10 @@ lfLens &lfLens::operator = (const lfLens &other)
     if (other.CalibFov)
         for (int i = 0; other.CalibFov [i]; i++)
             AddCalibFov (other.CalibFov [i]);
+    lf_free (CalibRealFocal); CalibRealFocal = NULL;
+    if (other.CalibRealFocal)
+        for (int i = 0; other.CalibRealFocal [i]; i++)
+            AddCalibRealFocal (other.CalibRealFocal [i]);
 
     return *this;
 }
@@ -201,10 +202,10 @@ void lfLens::GuessParameters ()
 
     char *old_numeric = setlocale (LC_NUMERIC, NULL);
     old_numeric = strdup (old_numeric);
-    setlocale (LC_NUMERIC,"C");
+    setlocale (LC_NUMERIC, "C");
 
     if (!MinAperture || !MinFocal)
-        _lf_parse_lens_name (Model, minf, maxf, mina, maxa);
+        _lf_parse_lens_name (Model, minf, maxf, mina);
 
     if (!MinAperture || !MinFocal)
     {
@@ -259,6 +260,15 @@ void lfLens::GuessParameters ()
                 if (f > maxf)
                     maxf = f;
             }
+        if (CalibRealFocal)
+            for (int i=0; CalibRealFocal [i]; i++)
+            {
+                float f = CalibRealFocal [i]->Focal;
+                if (f < minf)
+                    minf = f;
+                if (f > maxf)
+                    maxf = f;
+            }
 
     }
 
@@ -273,8 +283,6 @@ void lfLens::GuessParameters ()
 
     if (!MaxFocal)
         MaxFocal = MinFocal;
-    if (!MaxAperture)
-        MaxAperture = MinAperture;
 
     setlocale (LC_NUMERIC, old_numeric);
     free (old_numeric);
@@ -284,8 +292,9 @@ bool lfLens::Check ()
 {
     GuessParameters ();
 
-    if (!Model || !Mounts ||
-        MinFocal > MaxFocal || MinAperture > MaxAperture)
+    if (!Model || !Mounts || CropFactor <= 0 ||
+        MinFocal > MaxFocal || (MaxAperture && MinAperture > MaxAperture) ||
+        !isfinite (AspectRatio) || AspectRatio < 1)
         return false;
 
     return true;
@@ -302,9 +311,6 @@ const char *lfLens::GetDistortionModelDesc (
     static const lfParameter param_poly5_k2 = { "k2", -0.2F, 0.2F, 0.0F };
     static const lfParameter *param_poly5 [] = { &param_poly3_k1, &param_poly5_k2, NULL };
 
-    static const lfParameter param_fov1_omega = { "omega", 0.0F, 1.4F, 0.0F };
-    static const lfParameter *param_fov1 [] = { &param_fov1_omega, NULL };
-
     static const lfParameter param_ptlens_a = { "a", -0.2F, 0.2F, 0.0F };
     static const lfParameter param_ptlens_b = { "b", -0.2F, 0.2F, 0.0F };
     static const lfParameter param_ptlens_c = { "c", -0.2F, 0.2F, 0.0F };
@@ -322,7 +328,7 @@ const char *lfLens::GetDistortionModelDesc (
 
         case LF_DIST_MODEL_POLY3:
             if (details)
-                *details = "Ru = Rd * (1 - k1 + k1 * Rd^2)\n"
+                *details = "Rd = Ru * (1 - k1 + k1 * Ru^2)\n"
                     "Ref: http://www.imatest.com/docs/distortion.html";
             if (params)
                 *params = param_poly3;
@@ -330,23 +336,15 @@ const char *lfLens::GetDistortionModelDesc (
 
         case LF_DIST_MODEL_POLY5:
             if (details)
-                *details = "Ru = Rd * (1 + k1 * Rd^2 + k2 * Rd^4)\n"
+                *details = "Rd = Ru * (1 + k1 * Ru^2 + k2 * Ru^4)\n"
                     "Ref: http://www.imatest.com/docs/distortion.html";
             if (params)
                 *params = param_poly5;
             return "5th order polynomial";
 
-        case LF_DIST_MODEL_FOV1:
-            if (details)
-                *details = "Ru = tg (Rd * omega) / (2 * tg (omega/2))\n"
-                    "Ref: ftp://ftp-sop.inria.fr/chir/publis/devernay-faugeras:01.pdf";
-            if (params)
-                *params = param_fov1;
-            return "1st-order field-of-view";
-
         case LF_DIST_MODEL_PTLENS:
             if (details)
-                *details = "Ru = Rd * (a * Rd^3 + b * Rd^2 + c * Rd + 1 - (a + b + c))\n"
+                *details = "Rd = Ru * (a * Ru^3 + b * Ru^2 + c * Ru + 1 - (a + b + c))\n"
                     "Ref: http://wiki.panotools.org/Lens_correction_model";
             if (params)
                 *params = param_ptlens;
@@ -719,6 +717,35 @@ bool lfLens::RemoveCalibFov (int idx)
     return _lf_delobj (x.arr, idx);
 }
 
+static bool cmp_real_focal (const void *x1, const void *x2)
+{
+    const lfLensCalibRealFocal *d1 = static_cast<const lfLensCalibRealFocal *> (x1);
+    const lfLensCalibRealFocal *d2 = static_cast<const lfLensCalibRealFocal *> (x2);
+    return (d1->Focal == d2->Focal);
+}
+
+void lfLens::AddCalibRealFocal (const lfLensCalibRealFocal *lcf)
+{
+    // Avoid "dereferencing type-punned pointer will break strict-aliasing rules" warning
+    union
+    {
+        lfLensCalibRealFocal ***cd;
+        void ***arr;
+    } x = { &CalibRealFocal };
+    _lf_addobj (x.arr, lcf, sizeof (*lcf), cmp_real_focal);
+}
+
+bool lfLens::RemoveCalibRealFocal (int idx)
+{
+    // Avoid "dereferencing type-punned pointer will break strict-aliasing rules" warning
+    union
+    {
+        lfLensCalibRealFocal ***cd;
+        void ***arr;
+    } x = { &CalibRealFocal };
+    return _lf_delobj (x.arr, idx);
+}
+
 static int __insert_spline (void **spline, float *spline_dist, float dist, void *val)
 {
     if (dist < 0)
@@ -783,7 +810,7 @@ bool lfLens::InterpolateDistortion (float focal, lfLensCalibDistortion &res) con
             dm = c->Model;
         else if (dm != c->Model)
         {
-            g_warning ("WARNING: lens %s/%s has multiple distortion models defined\n",
+            g_warning ("[lensfun] lens %s/%s has multiple distortion models defined\n",
                        Maker, Model);
             continue;
         }
@@ -851,7 +878,7 @@ bool lfLens::InterpolateTCA (float focal, lfLensCalibTCA &res) const
             tcam = c->Model;
         else if (tcam != c->Model)
         {
-            g_warning ("WARNING: lens %s/%s has multiple TCA models defined\n",
+            g_warning ("[lensfun] lens %s/%s has multiple TCA models defined\n",
                        Maker, Model);
             continue;
         }
@@ -928,6 +955,8 @@ bool lfLens::InterpolateVignetting (
     for (size_t i = 0; i < ARRAY_LEN (res.Terms); i++)
         res.Terms [i] = 0;
 
+    // Use http://en.wikipedia.org/wiki/Inverse_distance_weighting with
+    // p = 3.5.
     float total_weighting = 0;
     const float power = 3.5;
 
@@ -943,7 +972,7 @@ bool lfLens::InterpolateVignetting (
         } 
         else if (vm != c->Model)
         {
-            g_warning ("WARNING: lens %s/%s has multiple vignetting models defined\n",
+            g_warning ("[lensfun] lens %s/%s has multiple vignetting models defined\n",
                        Maker, Model);
             continue;
         }
@@ -998,7 +1027,7 @@ bool lfLens::InterpolateCrop (float focal, lfLensCalibCrop &res) const
             cm = c->CropMode;
         else if (cm != c->CropMode)
         {
-            g_warning ("WARNING: lens %s/%s has multiple crop modes defined\n",
+            g_warning ("[lensfun] lens %s/%s has multiple crop modes defined\n",
                        Maker, Model);
             continue;
         }
@@ -1102,28 +1131,70 @@ bool lfLens::InterpolateFov (float focal, lfLensCalibFov &res) const
     return true;
 }
 
-gint _lf_lens_compare (gconstpointer a, gconstpointer b)
+bool lfLens::InterpolateRealFocal (float focal, lfLensCalibRealFocal &res) const
 {
-    int i;
-    lfLens *i1 = (lfLens *)a;
-    lfLens *i2 = (lfLens *)b;
+    if (!CalibRealFocal)
+        return false;
 
-    int cmp = _lf_strcmp (i1->Maker, i2->Maker);
-    if (cmp != 0)
-        return cmp;
-
-    cmp = _lf_strcmp (i1->Model, i2->Model);
-    if (cmp != 0)
-        return cmp;
-
-    for (i = 0; i1->Mounts [i] || i2->Mounts [i]; i++)
+    union
     {
-        cmp = _lf_strcmp (i1->Mounts [i], i2->Mounts [i]);
-        if (cmp != 0)
-            return cmp;
+        lfLensCalibRealFocal *spline [4];
+        void *spline_ptr [4];
+    };
+    float spline_dist [4] = { -FLT_MAX, -FLT_MAX, FLT_MAX, FLT_MAX };
+
+    memset (spline, 0, sizeof (spline));
+    int counter=0;
+    for (int i = 0; CalibRealFocal [i]; i++)
+    {
+        lfLensCalibRealFocal *c = CalibRealFocal [i];
+        if (c->RealFocal == 0)
+            continue;
+
+        counter++;
+        float df = focal - c->Focal;
+        if (df == 0.0)
+        {
+            // Exact match found, don't care to interpolate
+            res = *c;
+            return true;
+        }
+
+        __insert_spline (spline_ptr, spline_dist, df, c);
     }
 
-    cmp = int ((i1->MinFocal - i2->MinFocal) * 100);
+    //no valid data found
+    if (counter==0)
+        return false;
+
+    if (!spline [1] || !spline [2])
+    {
+        if (spline [1])
+            res = *spline [1];
+        else if (spline [2])
+            res = *spline [2];
+        else
+            return false;
+
+        return true;
+    }
+
+    // No exact match found, interpolate the model parameters
+    res.Focal = focal;
+
+    float t = (focal - spline [1]->Focal) / (spline [2]->Focal - spline [1]->Focal);
+
+    res.RealFocal = _lf_interpolate (
+        spline [0] ? spline [0]->RealFocal : FLT_MAX,
+        spline [1]->RealFocal, spline [2]->RealFocal,
+        spline [3] ? spline [3]->RealFocal : FLT_MAX, t);
+
+    return true;
+}
+
+gint _lf_lens_parameters_compare (const lfLens *i1, const lfLens *i2)
+{
+    int cmp = int ((i1->MinFocal - i2->MinFocal) * 100);
     if (cmp != 0)
         return cmp;
 
@@ -1131,15 +1202,29 @@ gint _lf_lens_compare (gconstpointer a, gconstpointer b)
     if (cmp != 0)
         return cmp;
 
-    cmp = int ((i1->MinAperture - i2->MinAperture) * 100);
-    if (cmp != 0)
-        return cmp;
+    return int ((i1->MinAperture - i2->MinAperture) * 100);
 
     // MaxAperture is usually not given in database...
     // so it's a guessed value, often incorrect.
-    //cmp = int ((i1->MaxAperture - i2->MaxAperture) * 100);
-    //if (cmp != 0)
-    //    return cmp;
+}
+
+gint _lf_lens_name_compare (const lfLens *i1, const lfLens *i2)
+{
+    int cmp = _lf_strcmp (i1->Maker, i2->Maker);
+    if (cmp != 0)
+        return cmp;
+
+    return _lf_strcmp (i1->Model, i2->Model);
+}
+
+gint _lf_lens_compare (gconstpointer a, gconstpointer b)
+{
+    lfLens *i1 = (lfLens *)a;
+    lfLens *i2 = (lfLens *)b;
+
+    int cmp = _lf_lens_name_compare (i1, i2);
+    if (cmp != 0)
+        return cmp;
 
     return int ((i1->CropFactor - i2->CropFactor) * 100);
 }
@@ -1166,7 +1251,7 @@ int _lf_lens_compare_score (const lfLens *pattern, const lfLens *match,
         if (pattern->Type != match->Type)
             return 0;
 
-    if (pattern->CropFactor > 0.01 && pattern->CropFactor < match->CropFactor - 0.01)
+    if (pattern->CropFactor > 0.01 && pattern->CropFactor < match->CropFactor * 0.96)
         return 0;
 
     if (pattern->CropFactor >= match->CropFactor * 1.41)
@@ -1179,6 +1264,10 @@ int _lf_lens_compare_score (const lfLens *pattern, const lfLens *match,
         score += 8;
     else if (pattern->CropFactor >= match->CropFactor * 1.01)
         score += 10;
+    else if (pattern->CropFactor >= match->CropFactor)
+        score += 5;
+    else if (pattern->CropFactor >= match->CropFactor * 0.96)
+        score += 3;
 
     switch (_lf_compare_num (pattern->MinFocal, match->MinFocal))
     {
@@ -1210,79 +1299,56 @@ int _lf_lens_compare_score (const lfLens *pattern, const lfLens *match,
             break;
     }
 
-    // MaxAperture is usually not given in database...
-    // so it's a guessed value, often incorrect.
     switch (_lf_compare_num (pattern->MaxAperture, match->MaxAperture))
     {
-        //case -1:
-        //    return 0;
+        case -1:
+            return 0;
 
         case +1:
             score += 10;
             break;
     }
 
-    // Check the lens mount, if specified
-    if (match->Mounts)
+    switch (_lf_compare_num (pattern->AspectRatio, match->AspectRatio))
     {
-        int old_score = score;
+        case -1:
+            return 0;
 
-        if (compat_mounts && !compat_mounts [0])
-            compat_mounts = NULL;
+        case +1:
+            score += 10;
+            break;
+    }
+
+    if (compat_mounts && !compat_mounts [0])
+        compat_mounts = NULL;
+
+    // Check the lens mount, if specified
+    if (match->Mounts && (pattern->Mounts || compat_mounts))
+    {
+        bool matching_mount_found = false;
 
         if (pattern->Mounts)
-        {
-            int nm = 0;
             for (int i = 0; pattern->Mounts [i]; i++)
                 for (int j = 0; match->Mounts [j]; j++)
                     if (!_lf_strcmp (pattern->Mounts [i], match->Mounts [j]))
                     {
-                        nm++;
-                        break;
+                        matching_mount_found = true;
+                        score += 10;
+                        goto exit_mount_search;
                     }
 
-            if (nm)
-            {
-                // Count number of mounts in the match lens
-                int match_nm;
-                for (match_nm = 0; match->Mounts [match_nm]; match_nm++)
-                    ;
-                // Don't allow 0 score, make it at least 1
-                int _score = (nm * 20) / match_nm;
-                if (!_score)
-                    _score = 1;
-                score += _score;
-            }
-        }
-
         if (compat_mounts)
-        {
-            int nm = 0;
             for (int i = 0; compat_mounts [i]; i++)
                 for (int j = 0; match->Mounts [j]; j++)
                     if (!_lf_strcmp (compat_mounts [i], match->Mounts [j]))
                     {
-                        nm++;
-                        break;
+                        matching_mount_found = true;
+                        score += 5;
+                        goto exit_mount_search;
                     }
 
-            if (nm)
-            {
-                // Count number of compatible mounts in the match lens
-                int match_nm;
-                for (match_nm = 0; compat_mounts [match_nm]; match_nm++)
-                    ;
-
-                // Don't allow 0 score, make it at least 1
-                int _score = (nm * 10) / match_nm;
-                if (!_score)
-                    _score = 1;
-                score += _score;
-            }
-        }
-
-        // If there were no mount matches, fail the comparison
-        if (old_score == score && (pattern->Mounts || compat_mounts))
+    exit_mount_search:
+        if (!matching_mount_found)
             return 0;
     }
 
@@ -1395,6 +1461,12 @@ cbool lf_lens_interpolate_fov (const lfLens *lens, float focal,
     return lens->InterpolateFov (focal, *res);
 }
 
+cbool lf_lens_interpolate_real_focal (const lfLens *lens, float focal,
+    lfLensCalibRealFocal *res)
+{
+    return lens->InterpolateRealFocal (focal, *res);
+}
+
 void lf_lens_add_calib_distortion (lfLens *lens, const lfLensCalibDistortion *dc)
 {
     lens->AddCalibDistortion (dc);
@@ -1446,3 +1518,13 @@ cbool lf_lens_remove_calib_fov (lfLens *lens, int idx)
     return lens->RemoveCalibFov (idx);
 }
 
+void lf_lens_add_calib_real_focal (lfLens *lens, const lfLensCalibRealFocal *lcf)
+{
+    lens->AddCalibRealFocal (lcf);
+}
+
+cbool lf_lens_remove_calib_real_focal (lfLens *lens, int idx)
+{
+    return lens->RemoveCalibRealFocal (idx);
+}
+
