@@ -32,7 +32,7 @@ typedef double lf_f64;
 class lfFuzzyStrCmp;
 
 /**
- * Return the absolute value of a number.
+ * @brief Return the absolute value of a number.
  * @param x
  *     A number
  * @return
@@ -44,7 +44,7 @@ template<typename T> static inline T absolute (T x)
 }
 
 /**
- * Return the square of the argument.
+ * @brief Return the square of the argument.
  * @param x
  *     A floating-point number.
  */
@@ -54,7 +54,7 @@ template<typename T> static inline T square (T x)
 }
 
 /**
- * Clamp a double value between 0 and max, then convert to given type.
+ * @brief Clamp a double value between 0 and max, then convert to given type.
  * @param x
  *     The number to clamp.
  * @param min
@@ -74,14 +74,14 @@ template<typename T> static inline T clampd (double x, double min, double max)
 }
 
 /**
- * Free a list of pointers.
+ * @brief Free a list of pointers.
  * @param list
  *     A NULL-terminated list of pointers
  */
 extern void _lf_list_free (void **list);
 
 /**
- * Make a copy of given value into given variable using g_strdup,
+ * @brief Make a copy of given value into given variable using g_strdup,
  * freeing the old value if defined.
  * @param var
  *     The variable to copy value into
@@ -91,7 +91,7 @@ extern void _lf_list_free (void **list);
 extern void _lf_setstr (gchar **var, const gchar *val);
 
 /**
- * Add a string to the end of a string list.
+ * @brief Add a string to the end of a string list.
  * @param var
  *     A pointer to an array of strings.
  * @param val
@@ -100,7 +100,8 @@ extern void _lf_setstr (gchar **var, const gchar *val);
 extern void _lf_addstr (gchar ***var, const gchar *val);
 
 /**
- * Insert a item into a GPtrArray, keeping the array sorted.
+ * @brief Insert a item into a GPtrArray, keeping the array sorted.
+ *
  * This method assumes that the array is already sorted, so
  * function uses a binary search algorithm.
  * Returns the index at which the item as inserted.
@@ -117,8 +118,8 @@ extern int _lf_ptr_array_insert_sorted (
     GPtrArray *array, void *item, GCompareFunc compare);
 
 /**
- * Same as _lf_ptr_array_insert_sorted(), but if array contains
- * a item equal to the inserted one, the new item overrides the old.
+ * @brief Insert a item into a GPtrArray, keeping the array sorted.  If array
+ * contains a item equal to the inserted one, the new item overrides the old.
  * @param array
  *     The array of pointers to similar items.
  * @param item
@@ -134,7 +135,8 @@ extern int _lf_ptr_array_insert_unique (
     GPtrArray *array, void *item, GCompareFunc compare, GDestroyNotify dest);
 
 /**
- * Find a item in a sorted array.
+ * @brief Find a item in a sorted array.
+ *
  * The function uses a binary search algorithm.
  * @param array
  *     The array of pointers to similar items.
@@ -147,7 +149,9 @@ extern int _lf_ptr_array_find_sorted (
     const GPtrArray *array, void *item, GCompareFunc compare);
 
 /**
- * Add a object to a list of objects. Accepts an optional pointer to
+ * @brief Add a object to a list of objects.
+ *
+ * Accepts an optional pointer to
  * a function that compares two values from the list, if function
  * returns true the existing object is replaced with the new one.
  * @param var
@@ -164,7 +168,7 @@ extern void _lf_addobj (void ***var, const void *val, size_t val_size,
     bool (*cmpf) (const void *, const void *));
 
 /**
- * Remove an object from a list of objects, freeing memory which was
+ * @brief Remove an object from a list of objects, freeing memory which was
  * allocated by _lf_addobj().
  * @param var
  *     A pointer to an array of objects.
@@ -176,7 +180,7 @@ extern void _lf_addobj (void ***var, const void *val, size_t val_size,
 extern bool _lf_delobj (void ***var, int idx);
 
 /**
- * Appends a formatted string to a dynamically-growing string
+ * @brief Appends a formatted string to a dynamically-growing string
  * using g_markup_printf_escaped() internally.
  * @param output
  *     The output array.
@@ -186,7 +190,7 @@ extern bool _lf_delobj (void ***var, int idx);
 extern void _lf_xml_printf (GString *output, const char *format, ...);
 
 /**
- * Output a multi-language value to output string.
+ * @brief Output a multi-language value to output string.
  *
  * Outputs a number of lines which looks like:
  *
@@ -208,14 +212,15 @@ extern void _lf_xml_printf_mlstr (GString *output, const char *prefix,
                                   const char *element, const lfMLstr val);
 
 /**
- * Get the XML id of the given distortion model
+ * @brief Get the XML id of the given distortion model
  * @param model
  *     The model.
  */
 extern const char *_lf_get_distortion_model_id (lfDistortionModel model);
 
 /**
- * Something like a very advanced strcmp().
+ * @brief Something like a very advanced strcmp().
+ *
  * It doesn't segfault if one or both strings are NULL:
  * NULL is considered to be less than any string.
  * Actually this function does a fuzzy comparison of the strings,
@@ -225,14 +230,19 @@ extern const char *_lf_get_distortion_model_id (lfDistortionModel model);
 extern int _lf_strcmp (const char *s1, const char *s2);
 
 /**
- * Same as _lf_strcmp(), but compares a string with a multi-language
- * string. If it equals any of the translations, 0 is returned, otherwise
+ * @brief Same as _lf_strcmp(), but compares a string with a multi-language
+ * string.
+ *
+ * If it equals any of the translations, 0 is returned, otherwise
  * the result of strcmp() with the first (default) string is returned.
  */
 extern int _lf_mlstrcmp (const char *s1, const lfMLstr s2);
 
 /**
- * Comparison function for mount sorting and finding.
+ * @brief Comparison function for mount sorting and finding.
+ *
+ * Since this function is used when reading the database, it effectively
+ * enforces the primary key for mounts, which is their Name.
  * @param a
  *     A pointer to first lfMount object.
  * @param b
@@ -243,7 +253,11 @@ extern int _lf_mlstrcmp (const char *s1, const lfMLstr s2);
 extern gint _lf_mount_compare (gconstpointer a, gconstpointer b);
 
 /**
- * Comparison function for camera sorting and finding.
+ * @brief Comparison function for camera sorting and finding.
+ *
+ * Since this function is used when reading the database, it effectively
+ * enforces the primary key for cameras, which is the combination of the
+ * attributes Maker, Model, and Variant.
  * @param a
  *     A pointer to first lfCamera object.
  * @param b
@@ -254,7 +268,44 @@ extern gint _lf_mount_compare (gconstpointer a, gconstpointer b);
 extern gint _lf_camera_compare (gconstpointer a, gconstpointer b);
 
 /**
- * Comparison function for lens sorting and finding.
+ * @brief Comparison helper function for lens sorting and finding.
+ *
+ * This function compares the numerical parameters of the lenses: MinFocal,
+ * MaxFocal, and MinAperture, in this order.  Since it is not meant to be used
+ * as a sorting key function directly, it doesn't take generic pointers as
+ * parameters.  Instead, it is supposed to be used by such sorting key
+ * functions like _lf_lens_compare.
+ * @param i1
+ *     A pointer to first lfLens object.
+ * @param i2
+ *     A pointer to second lfLens object.
+ * @return
+ *     Positive if i1 > i2, negative if i1 < i2, zero if they are equal.
+ */
+extern gint _lf_lens_parameters_compare (const lfLens *i1, const lfLens *i2);
+
+/**
+ * @brief Comparison helper function for lens sorting and finding.
+ *
+ * This function compares the names of the lenses: Maker and Model, in this
+ * order.  Since it is not meant to be used as a sorting key function directly,
+ * it doesn't take generic pointers as parameters.  Instead, it is supposed to
+ * be used by such sorting key functions like _lf_lens_compare.
+ * @param i1
+ *     A pointer to first lfLens object.
+ * @param i2
+ *     A pointer to second lfLens object.
+ * @return
+ *     Positive if i1 > i2, negative if i1 < i2, zero if they are equal.
+ */
+extern gint _lf_lens_name_compare (const lfLens *i1, const lfLens *i2);
+
+/**
+ * @brief Comparison function for lens sorting and finding.
+ *
+ * Since this function is used when reading the database, it effectively
+ * enforces the primary key for lenses, which is the combination of the
+ * attributes Maker, Model, and CropFactor.
  * @param a
  *     A pointer to first lfLens object.
  * @param b
@@ -265,7 +316,8 @@ extern gint _lf_camera_compare (gconstpointer a, gconstpointer b);
 extern gint _lf_lens_compare (gconstpointer a, gconstpointer b);
 
 /**
- * Get a interpolated value.
+ * @brief Get an interpolated value.
+ *
  * Currently this uses a kind of Catmull-Rom splines with linear
  * interpolation at the ends, allowing for non-evenly spaced values
  * and handling the extreme cases correctly (the one at the start
@@ -288,7 +340,24 @@ extern gint _lf_lens_compare (gconstpointer a, gconstpointer b);
 extern float _lf_interpolate (float y1, float y2, float y3, float y4, float t);
 
 /**
- * Compare a lens with a pattern and return a matching score.
+ * @brief Scans for the timestamp of a lensfun database.
+ *
+ * "database" means here a directory containing XML files.  If it also contains
+ * a file timestamp.txt, its content is interpreted as a string-encoded
+ * floating point value, representing the timestamp in seconds since the Epoch
+ * (UNIX time).  If it is not present, 0 is returned.  If the directory is not
+ * found or empty, a negative value is returned.
+ * @param dirname
+ *     the name of the directory containing a lensfun database
+ * @return
+ *     the number of seconds since the Epoch when this database was last
+ *     updated.
+ */
+extern long int _lf_read_database_timestamp(const gchar *dirname);
+
+/**
+ * @brief Compare a lens with a pattern and return a matching score.
+ *
  * The comparison is quasi-intelligent: the order of words in a name
  * does not matter; the more words from match are present in the pattern,
  * the higher is score. Numeric parameters have to coincide or not be specified
@@ -329,14 +398,15 @@ enum
 };
 
 /**
- * Detect supported CPU features (used for runtime selection of accelerated
+ * @brief Detect supported CPU features (used for runtime selection of accelerated
  * functions for specific architecture extensions).
  */
 extern guint _lf_detect_cpu_features ();
 
 /**
- * Google-in-your-pocket: a fuzzy string comparator.
- * This has been designed for comparing lens model names.
+ * @brief Google-in-your-pocket: a fuzzy string comparator.
+ *
+ * This has been designed for comparing lens and camera model names.
  * At construction the pattern is split into words and then the component
  * words from target are matched against them.
  */
@@ -353,6 +423,7 @@ public:
     /**
      * @param pattern
      *     The pattern which will be compared against a number of strings.
+     *     This is typically what was found in the EXIF data.
      * @param allwords
      *     If true, all words of the pattern must be present in the
      *     target string. If not, a looser result will be accepted,
@@ -362,30 +433,43 @@ public:
     ~lfFuzzyStrCmp ();
 
     /**
-     * Fuzzy compare the pattern with a string.
+     * @brief Fuzzy compare the pattern with a string.
      * @param match
-     *     The string to match against.
+     *     The string to match against.  This is typically taken from the
+     *     Lensfun database.
      * @return
-     *     Returns a score in range 0-100, depending on the number of
-     *     matched words.
+     *     Returns a score in range 0-100.  If the match succedes, this score
+     *     is the number of matched words divided by the mean word count of
+     *     pattern and string, given as a percentage.  If it fails, it is 0.
+     *     It fails if no words could be matched, of if allwords was set to
+     *     true and one word in pattern could not be found in match.
      */
     int Compare (const char *match);
 
     /**
-     * Compares the pattern with a multi-language string.
+     * @brief Compares the pattern with a multi-language string.
+     *
      * This function returns the largest score as compared against
      * every of the translated strings.
      * @param match
-     *     The multi-language string to match against.
+     *     The multi-language string to match against.  This is typically taken
+     *     from the Lensfun database.
      * @return
-     *     Returns a score in range 0-100, depending on the number of
-     *     matched words.
+     *     Returns the maximal score in range 0-100.  For every component of
+     *     the multi-language string, a score is computed: If the match
+     *     succedes, the score is the number of matched words divided by the
+     *     mean word count of pattern and string, given as a percentage.  If it
+     *     fails, it is 0.  It fails if no words could be matched, of if
+     *     allwords was set to true and one word in pattern could not be found
+     *     in match.
      */
     int Compare (const lfMLstr match);
 };
 
 /**
- * The real Database class, with some additional data fields
+ * @brief The real Database class.
+ *
+ * It contains some additional data fields
  * which must not be seen from the application at all (even
  * private members don't qualify because we will have to include
  * glib.h from lens.h and that's something we want to hide).
@@ -396,6 +480,16 @@ struct lfExtDatabase : public lfDatabase
     GPtrArray *Cameras;
     GPtrArray *Lenses;
 
+    /**
+     * @brief Load all XML files from a directory.
+     *
+     * This is an internal function used by lfDatabase::Load ().  It ignores
+     * all errors.
+     * @param dirname
+     *     The directory to be read.
+     */
+    void LoadDirectory (const gchar *dirname);
+
     lfExtDatabase ()
     {
         HomeDataDir = g_build_filename (g_get_user_data_dir (),
@@ -454,11 +548,8 @@ struct lfColorCallbackData : public lfCallbackData
 };
 
 /**
- * This is the extended lfModifier class, with implementation details
+ * @brief This is the extended lfModifier class, with implementation details
  * hidden from the public header file.
- *
- * The order of image correction is always this: Un-TCA, Un-distort,
- * Un-vignette and finally Un-CCI.
  */
 struct lfExtModifier : public lfModifier
 {
@@ -474,6 +565,12 @@ struct lfExtModifier : public lfModifier
     double CenterX, CenterY;
     /// The coefficients for conversion to and from normalized coords
     double NormScale, NormUnScale;
+    /// Factor to transform from normalized into absolute coords (mm).  Needed
+    /// for geometry transformation.
+    double NormalizedInMillimeters;
+    /// Used for conversion from distortion to vignetting coordinate system of
+    /// the calibration sensor
+    double AspectRatioCorrection;
 
     /// A list of subpixel coordinate modifier callbacks.
     GPtrArray *SubpixelCallbacks;
@@ -482,22 +579,23 @@ struct lfExtModifier : public lfModifier
     /// A list of pixel coordinate modifier callbacks.
     GPtrArray *CoordCallbacks;
 
-    static void ModifyCoord_TCA_Linear (void *data, float *iocoord, int count);
     static void ModifyCoord_UnTCA_Linear (void *data, float *iocoord, int count);
-    static void ModifyCoord_TCA_Poly3 (void *data, float *iocoord, int count);
+    static void ModifyCoord_TCA_Linear (void *data, float *iocoord, int count);
     static void ModifyCoord_UnTCA_Poly3 (void *data, float *iocoord, int count);
+    static void ModifyCoord_TCA_Poly3 (void *data, float *iocoord, int count);
 
-    static void ModifyCoord_Dist_Poly3 (void *data, float *iocoord, int count);
     static void ModifyCoord_UnDist_Poly3 (void *data, float *iocoord, int count);
+    static void ModifyCoord_Dist_Poly3 (void *data, float *iocoord, int count);
+#ifdef VECTORIZATION_SSE
+    static void ModifyCoord_Dist_Poly3_SSE (void *data, float *iocoord, int count);
+#endif
+    static void ModifyCoord_UnDist_Poly5 (void *data, float *iocoord, int count);
     static void ModifyCoord_Dist_Poly5 (void *data, float *iocoord, int count);
-    static void ModifyCoord_UnDist_Poly5 (void *data, float *iocoord, int count);
-    static void ModifyCoord_Dist_FOV1 (void *data, float *iocoord, int count);
-    static void ModifyCoord_UnDist_FOV1 (void *data, float *iocoord, int count);
+    static void ModifyCoord_UnDist_PTLens (void *data, float *iocoord, int count);
     static void ModifyCoord_Dist_PTLens (void *data, float *iocoord, int count);
-    static void ModifyCoord_UnDist_PTLens (void *data, float *iocoord, int count);
 #ifdef VECTORIZATION_SSE
-    static void ModifyCoord_Dist_PTLens_SSE (void *data, float *iocoord, int count);
     static void ModifyCoord_UnDist_PTLens_SSE (void *data, float *iocoord, int count);
+    static void ModifyCoord_Dist_PTLens_SSE (void *data, float *iocoord, int count);
 #endif
     static void ModifyCoord_Geom_Rect_FishEye (void *data, float *iocoord, int count);
     static void ModifyCoord_Geom_Rect_Panoramic (void *data, float *iocoord, int count);
