@@ -14,7 +14,7 @@
 #include "lensfunprv.h"
 #include <xmmintrin.h>
 
-void lfExtModifier::ModifyCoord_Dist_PTLens_SSE (void *data, float *iocoord, int count)
+void lfExtModifier::ModifyCoord_UnDist_PTLens_SSE (void *data, float *iocoord, int count)
 {
   float *param = (float *)data;
 
@@ -34,42 +34,42 @@ void lfExtModifier::ModifyCoord_Dist_PTLens_SSE (void *data, float *iocoord, int
     __m128 c1 = _mm_loadu_ps (&iocoord[8*i+4]);
     __m128 x = _mm_shuffle_ps (c0, c1, _MM_SHUFFLE (2, 0, 2, 0));
     __m128 y = _mm_shuffle_ps (c0, c1, _MM_SHUFFLE (3, 1, 3, 1));
-    __m128 ru = _mm_add_ps (_mm_mul_ps (x, x), _mm_mul_ps (y, y));
+    __m128 rd = _mm_add_ps (_mm_mul_ps (x, x), _mm_mul_ps (y, y));
 
     // We don't check for zero, but set it to a very small value instead
-    ru = _mm_max_ps (ru, very_small);
-    ru = _mm_rcp_ps (_mm_rsqrt_ps (ru));
-    __m128 rd = ru;
+    rd = _mm_max_ps (rd, very_small);
+    rd = _mm_rcp_ps (_mm_rsqrt_ps (rd));
+    __m128 ru = rd;
     for (int step = 0; step < 4; step++)
     {
-      // frd = rd * (a * rd^2 * rd + b * rd^2 + c * rd + d) - ru
-      __m128 rd_sq =  _mm_mul_ps (rd, rd);
-      __m128 frd = _mm_mul_ps (_mm_mul_ps (a, rd), rd_sq);
-      __m128 t = _mm_add_ps (_mm_mul_ps (b, rd_sq), _mm_add_ps (d, _mm_mul_ps (c, rd)));
-      frd = _mm_sub_ps (_mm_mul_ps (_mm_add_ps (t, frd), rd), ru);
+      // fru = ru * (a * ru^2 * ru + b * ru^2 + c * ru + d) - rd
+      __m128 ru_sq =  _mm_mul_ps (ru, ru);
+      __m128 fru = _mm_mul_ps (_mm_mul_ps (a, ru), ru_sq);
+      __m128 t = _mm_add_ps (_mm_mul_ps (b, ru_sq), _mm_add_ps (d, _mm_mul_ps (c, ru)));
+      fru = _mm_sub_ps (_mm_mul_ps (_mm_add_ps (t, fru), ru), rd);
 
       // This is most likely faster than loading form L1 cache
       __m128 two = _mm_add_ps (one, one);
       __m128 three = _mm_add_ps (one, two);
       __m128 four = _mm_add_ps (two, two);
 
-      // corr =  4 * a * rd * rd^2 + 3 * b * rd^2 + 2 * c * rd + d
-      __m128 corr = _mm_mul_ps (c, rd);
+      // corr =  4 * a * ru * ru^2 + 3 * b * ru^2 + 2 * c * ru + d
+      __m128 corr = _mm_mul_ps (c, ru);
       corr = _mm_add_ps (d, _mm_add_ps (corr, corr));
-      t = _mm_mul_ps (rd_sq, _mm_mul_ps (three, b));
-      corr = _mm_add_ps (corr, _mm_mul_ps (_mm_mul_ps (rd, rd_sq), _mm_mul_ps (four, a)));
+      t = _mm_mul_ps (ru_sq, _mm_mul_ps (three, b));
+      corr = _mm_add_ps (corr, _mm_mul_ps (_mm_mul_ps (ru, ru_sq), _mm_mul_ps (four, a)));
       corr = _mm_rcp_ps (_mm_add_ps (corr, t));
 
-      // rd -= frd * corr
-      rd = _mm_sub_ps (rd, _mm_mul_ps (frd, corr));
+      // ru -= fru * corr
+      ru = _mm_sub_ps (ru, _mm_mul_ps (fru, corr));
     }
     // We don't check for zero, but set it to a very small value instead
-    rd = _mm_max_ps (rd, very_small);
+    ru = _mm_max_ps (ru, very_small);
 
-    // rd /= ru
-    rd = _mm_mul_ps (rd, _mm_rcp_ps(ru));
-    c0 = _mm_mul_ps (c0, rd);
-    c1 = _mm_mul_ps (c1, rd);
+    // ru /= rd
+    ru = _mm_mul_ps (ru, _mm_rcp_ps(rd));
+    c0 = _mm_mul_ps (c0, ru);
+    c1 = _mm_mul_ps (c1, ru);
     _mm_storeu_ps (&iocoord [8 * i], c0);
     _mm_storeu_ps (&iocoord [8 * i + 4], c1);
   }
@@ -77,7 +77,7 @@ void lfExtModifier::ModifyCoord_Dist_PTLens_SSE (void *data, float *iocoord, int
   loop_count *= 4;
   int remain = count - loop_count;
   if (remain) 
-    ModifyCoord_Dist_PTLens (data, &iocoord [loop_count * 2], remain);
+    ModifyCoord_UnDist_PTLens (data, &iocoord [loop_count * 2], remain);
 }
 
 #if defined (_MSC_VER)
@@ -86,11 +86,11 @@ typedef size_t uintptr_t;
 typedef __SIZE_TYPE__ uintptr_t;
 #endif
 
-void lfExtModifier::ModifyCoord_UnDist_PTLens_SSE (void *data, float *iocoord, int count)
+void lfExtModifier::ModifyCoord_Dist_PTLens_SSE (void *data, float *iocoord, int count)
 {
   float *param = (float *)data;
 
-  // Ru = Rd * (a * Rd^3 + b * Rd^2 + c * Rd + d)
+  // Rd = Ru * (a * Ru^3 + b * Ru^2 + c * Ru + d)
   __m128 a = _mm_set_ps1 (param [0]);
   __m128 b = _mm_set_ps1 (param [1]);
   __m128 c = _mm_set_ps1 (param [2]);
@@ -106,13 +106,13 @@ void lfExtModifier::ModifyCoord_UnDist_PTLens_SSE (void *data, float *iocoord, i
       __m128 c1 = _mm_load_ps (&iocoord [8 * i + 4]);
       __m128 x = _mm_shuffle_ps (c0, c1, _MM_SHUFFLE (2, 0, 2, 0));
       __m128 y = _mm_shuffle_ps (c0, c1, _MM_SHUFFLE (3, 1, 3, 1));
-      __m128 r2 = _mm_add_ps (_mm_mul_ps (x, x), _mm_mul_ps (y, y));
-      __m128 r = _mm_rcp_ps (_mm_rsqrt_ps (r2));
+      __m128 ru2 = _mm_add_ps (_mm_mul_ps (x, x), _mm_mul_ps (y, y));
+      __m128 ru = _mm_rcp_ps (_mm_rsqrt_ps (ru2));
 
-      // Calculate poly3 = a * r2 * r + b * r2 + c * r + d;
-      __m128 t = _mm_mul_ps (r2, b);
-      __m128 poly3 = _mm_mul_ps (_mm_mul_ps (a, r2), r);
-      t = _mm_add_ps (t, _mm_mul_ps (r, c));
+      // Calculate poly3 = a * ru2 * ru + b * ru2 + c * ru + d;
+      __m128 t = _mm_mul_ps (ru2, b);
+      __m128 poly3 = _mm_mul_ps (_mm_mul_ps (a, ru2), ru);
+      t = _mm_add_ps (t, _mm_mul_ps (ru, c));
       poly3 = _mm_add_ps (t, _mm_add_ps (poly3, d));
       _mm_store_ps (&iocoord [8 * i], _mm_mul_ps (poly3, c0));
       _mm_store_ps (&iocoord [8 * i + 4], _mm_mul_ps (poly3, c1));
@@ -124,13 +124,13 @@ void lfExtModifier::ModifyCoord_UnDist_PTLens_SSE (void *data, float *iocoord, i
       __m128 c1 = _mm_loadu_ps (&iocoord [8 * i + 4]);
       __m128 x = _mm_shuffle_ps (c0, c1, _MM_SHUFFLE (2, 0, 2, 0));
       __m128 y = _mm_shuffle_ps (c0, c1, _MM_SHUFFLE (3, 1, 3, 1));
-      __m128 r2 = _mm_add_ps (_mm_mul_ps (x, x), _mm_mul_ps (y, y));
-      __m128 r = _mm_rcp_ps (_mm_rsqrt_ps (r2));
+      __m128 ru2 = _mm_add_ps (_mm_mul_ps (x, x), _mm_mul_ps (y, y));
+      __m128 ru = _mm_rcp_ps (_mm_rsqrt_ps (ru2));
 
-      // Calculate poly3 = a * r2 * r + b * r2 + c * r + d;
-      __m128 poly3 = _mm_mul_ps (_mm_mul_ps (a, r2), r);
-      __m128 t = _mm_mul_ps (r2, b);
-      poly3 = _mm_add_ps (poly3, _mm_mul_ps (r, c));
+      // Calculate poly3 = a * ru2 * ru + b * ru2 + c * ru + d;
+      __m128 poly3 = _mm_mul_ps (_mm_mul_ps (a, ru2), ru);
+      __m128 t = _mm_mul_ps (ru2, b);
+      poly3 = _mm_add_ps (poly3, _mm_mul_ps (ru, c));
       poly3 = _mm_add_ps (poly3, _mm_add_ps (t, d));
       _mm_storeu_ps (&iocoord [8 * i], _mm_mul_ps (poly3, c0));
       _mm_storeu_ps (&iocoord [8 * i + 4], _mm_mul_ps (poly3, c1));
@@ -139,7 +139,51 @@ void lfExtModifier::ModifyCoord_UnDist_PTLens_SSE (void *data, float *iocoord, i
   loop_count *= 4;
   int remain = count - loop_count;
   if (remain)
-    ModifyCoord_UnDist_PTLens (data, &iocoord [loop_count * 2], remain);
+    ModifyCoord_Dist_PTLens (data, &iocoord [loop_count * 2], remain);
+}
+
+void lfExtModifier::ModifyCoord_Dist_Poly3_SSE (void *data, float *iocoord, int count)
+{
+  float *param = (float *)data;
+
+  // Rd = Ru * (d + k1 * Ru^2),  d = 1 - k1
+  __m128 k1 = _mm_set_ps1 (param [0]);
+  __m128 d = _mm_sub_ps (_mm_set_ps1 (1.0f), k1);
+  int aligned = !((uintptr_t)(iocoord)&0xf);
+
+  // SSE Loop processes 4 pixels/loop
+  int loop_count = count / 4;
+  if (aligned)
+    for (int i = 0; i < loop_count ; i++)
+    {
+      __m128 c0 = _mm_load_ps (&iocoord [8 * i]);
+      __m128 c1 = _mm_load_ps (&iocoord [8 * i + 4]);
+      __m128 x = _mm_shuffle_ps (c0, c1, _MM_SHUFFLE (2, 0, 2, 0));
+      __m128 y = _mm_shuffle_ps (c0, c1, _MM_SHUFFLE (3, 1, 3, 1));
+
+      // Calculate poly3 = k1 * ru * ru + d;
+      __m128 poly3 = _mm_add_ps (_mm_mul_ps (_mm_add_ps (_mm_mul_ps (x, x), _mm_mul_ps (y, y)), k1), d);
+      _mm_store_ps (&iocoord [8 * i], _mm_mul_ps (poly3, c0));
+      _mm_store_ps (&iocoord [8 * i + 4], _mm_mul_ps (poly3, c1));
+    }
+  else
+    for (int i = 0; i < loop_count ; i++)
+    {
+      __m128 c0 = _mm_loadu_ps (&iocoord [8 * i]);
+      __m128 c1 = _mm_loadu_ps (&iocoord [8 * i + 4]);
+      __m128 x = _mm_shuffle_ps (c0, c1, _MM_SHUFFLE (2, 0, 2, 0));
+      __m128 y = _mm_shuffle_ps (c0, c1, _MM_SHUFFLE (3, 1, 3, 1));
+
+      // Calculate poly3 = k1 * ru * ru + d;
+      __m128 poly3 = _mm_add_ps (_mm_mul_ps (_mm_add_ps (_mm_mul_ps (x, x), _mm_mul_ps (y, y)), k1), d);
+      _mm_storeu_ps (&iocoord [8 * i], _mm_mul_ps (poly3, c0));
+      _mm_storeu_ps (&iocoord [8 * i + 4], _mm_mul_ps (poly3, c1));
+    }
+
+  loop_count *= 4;
+  int remain = count - loop_count;
+  if (remain)
+    ModifyCoord_Dist_Poly3 (data, &iocoord [loop_count * 2], remain);
 }
 
 #endif
