@@ -33,32 +33,23 @@ bool lfModifier::AddCoordCallbackDistortion (lfLensCalibDistortion &model, bool
                 if (!model.Terms [0])
                     return false;
                 tmp [0] = 1.0 / model.Terms [0];
-                AddCoordCallback (lfExtModifier::ModifyCoord_Dist_Poly3, 250,
+                AddCoordCallback (lfExtModifier::ModifyCoord_UnDist_Poly3, 250,
                                   tmp, sizeof (float));
                 break;
 
             case LF_DIST_MODEL_POLY5:
-                AddCoordCallback (lfExtModifier::ModifyCoord_Dist_Poly5, 250,
+                AddCoordCallback (lfExtModifier::ModifyCoord_UnDist_Poly5, 250,
                                   model.Terms, sizeof (float) * 2);
                 break;
 
-            case LF_DIST_MODEL_FOV1:
-                if (!model.Terms [0])
-                    return false;
-                tmp [0] = 1.0 / model.Terms [0];
-                tmp [1] = 2.0 * tan (model.Terms [0] / 2.0);
-                AddCoordCallback (lfExtModifier::ModifyCoord_Dist_FOV1, 250,
-                                  tmp, sizeof (float) * 2);
-                break;
-
             case LF_DIST_MODEL_PTLENS:
 #ifdef VECTORIZATION_SSE
                 if (_lf_detect_cpu_features () & LF_CPU_FLAG_SSE)
-                    AddCoordCallback (lfExtModifier::ModifyCoord_Dist_PTLens_SSE, 250,
+                    AddCoordCallback (lfExtModifier::ModifyCoord_UnDist_PTLens_SSE, 250,
                                       model.Terms, sizeof (float) * 3);
                 else
 #endif
-                AddCoordCallback (lfExtModifier::ModifyCoord_Dist_PTLens, 250,
+                AddCoordCallback (lfExtModifier::ModifyCoord_UnDist_PTLens, 250,
                                   model.Terms, sizeof (float) * 3);
                 break;
 
@@ -69,31 +60,29 @@ bool lfModifier::AddCoordCallbackDistortion (lfLensCalibDistortion &model, bool
         switch (model.Model)
         {
             case LF_DIST_MODEL_POLY3:
-                AddCoordCallback (lfExtModifier::ModifyCoord_UnDist_Poly3, 750,
+#ifdef VECTORIZATION_SSE
+                if (_lf_detect_cpu_features () & LF_CPU_FLAG_SSE)
+                    AddCoordCallback (lfExtModifier::ModifyCoord_Dist_Poly3_SSE, 750,
+                                      model.Terms, sizeof (float));
+                else
+#endif
+                AddCoordCallback (lfExtModifier::ModifyCoord_Dist_Poly3, 750,
                                   model.Terms, sizeof (float));
                 break;
 
             case LF_DIST_MODEL_POLY5:
-                AddCoordCallback (lfExtModifier::ModifyCoord_UnDist_Poly5, 750,
+                AddCoordCallback (lfExtModifier::ModifyCoord_Dist_Poly5, 750,
                                   model.Terms, sizeof (float) * 2);
                 break;
 
-            case LF_DIST_MODEL_FOV1:
-                if (!model.Terms [0])
-                    return false;
-                tmp [0] = model.Terms [0];
-                tmp [1] = 0.5 / tan (model.Terms [0] / 2.0);
-                AddCoordCallback (lfExtModifier::ModifyCoord_UnDist_FOV1, 750, tmp, sizeof (float) * 2);
-                break;
-
             case LF_DIST_MODEL_PTLENS:
 #ifdef VECTORIZATION_SSE
                 if (_lf_detect_cpu_features () & LF_CPU_FLAG_SSE)
-                    AddCoordCallback (lfExtModifier::ModifyCoord_UnDist_PTLens_SSE, 750,
+                    AddCoordCallback (lfExtModifier::ModifyCoord_Dist_PTLens_SSE, 750,
                                       model.Terms, sizeof (float) * 3);
                 else
 #endif
-                AddCoordCallback (lfExtModifier::ModifyCoord_UnDist_PTLens, 750,
+                AddCoordCallback (lfExtModifier::ModifyCoord_Dist_PTLens, 750,
                                   model.Terms, sizeof (float) * 3);
                 break;
 
@@ -215,10 +204,9 @@ bool lfModifier::AddCoordCallbackScale (float scale, bool reverse)
 
 bool lfModifier::AddCoordCallbackGeometry (lfLensType from, lfLensType to, float focal)
 {
+    lfExtModifier *This = static_cast<lfExtModifier *> (this);
     float tmp [2];
-    // The magic "12" is half of 36mm frame height
-    // (which is 1.0 in our normalized coordinates)
-    tmp [0] = focal / 12.0;
+    tmp [0] = focal / This->NormalizedInMillimeters;
     tmp [1] = 1.0 / tmp [0];
 
     if(from == to)
@@ -459,7 +447,7 @@ void lfExtModifier::ModifyCoord_Scale (void *data, float *iocoord, int count)
     }
 }
 
-void lfExtModifier::ModifyCoord_Dist_Poly3 (void *data, float *iocoord, int count)
+void lfExtModifier::ModifyCoord_UnDist_Poly3 (void *data, float *iocoord, int count)
 {
     const float inv_k1 = *(float *)data;
     const float one_minus_k1_div_k1 = (1 - 1.0 / inv_k1) * inv_k1;
@@ -468,48 +456,48 @@ void lfExtModifier::ModifyCoord_Dist_Poly3 (void *data, float *iocoord, int coun
     {
         float x = iocoord [0];
         float y = iocoord [1];
-        double ru = sqrt (x * x + y * y);
-        if (ru == 0.0)
+        double rd = sqrt (x * x + y * y);
+        if (rd == 0.0)
             continue;
 
-        float ru_div_k1 = ru * inv_k1;
+        float rd_div_k1 = rd * inv_k1;
 
         // Use Newton's method to avoid dealing with complex numbers
         // When carefully tuned this works almost as fast as Cardano's
         // method (and we don't use complex numbers in it, which is
         // required for a full solution!)
         //
-        // Original function: Ru = k1 * Rd^3 + (1 - k1) * Rd
-        // Target function:   k1 * Rd^3 + (1 - k1) * Rd - Ru = 0
-        // Divide by k1:      Rd^3 + Rd * (1 - k1)/k1 - Ru/k1 = 0
-        // Derivative:        3 * Rd^2 + (1 - k1)/k1
-        double rd = ru;
+        // Original function: Rd = k1 * Ru^3 + (1 - k1) * Ru
+        // Target function:   k1 * Ru^3 + (1 - k1) * Ru - Rd = 0
+        // Divide by k1:      Ru^3 + Ru * (1 - k1)/k1 - Rd/k1 = 0
+        // Derivative:        3 * Ru^2 + (1 - k1)/k1
+        double ru = rd;
         for (int step = 0; ; step++)
         {
-            double frd = rd * rd * rd + rd * one_minus_k1_div_k1 - ru_div_k1;
-            if (frd >= -NEWTON_EPS && frd < NEWTON_EPS)
+            double fru = ru * ru * ru + ru * one_minus_k1_div_k1 - rd_div_k1;
+            if (fru >= -NEWTON_EPS && fru < NEWTON_EPS)
                 break;
             if (step > 5)
                 // Does not converge, no real solution in this area?
                 goto next_pixel;
 
-            rd -= frd / (3 * rd * rd + one_minus_k1_div_k1);
+            ru -= fru / (3 * ru * ru + one_minus_k1_div_k1);
         }
-        if (rd < 0.0)
+        if (ru < 0.0)
             continue; // Negative radius does not make sense at all
 
-        rd /= ru;
-        iocoord [0] = x * rd;
-        iocoord [1] = y * rd;
+        ru /= rd;
+        iocoord [0] = x * ru;
+        iocoord [1] = y * ru;
 
     next_pixel:
         ;
     }
 }
 
-void lfExtModifier::ModifyCoord_UnDist_Poly3 (void *data, float *iocoord, int count)
+void lfExtModifier::ModifyCoord_Dist_Poly3 (void *data, float *iocoord, int count)
 {
-    // Ru = Rd * (1 - k1 + k1 * Rd^2)
+    // Rd = Ru * (1 - k1 + k1 * Ru^2)
     const float k1 = *(float *)data;
     const float one_minus_k1 = 1.0 - k1;
 
@@ -524,7 +512,7 @@ void lfExtModifier::ModifyCoord_UnDist_Poly3 (void *data, float *iocoord, int co
     }
 }
 
-void lfExtModifier::ModifyCoord_Dist_Poly5 (void *data, float *iocoord, int count)
+void lfExtModifier::ModifyCoord_UnDist_Poly5 (void *data, float *iocoord, int count)
 {
     float *param = (float *)data;
     float k1 = param [0];
@@ -534,40 +522,40 @@ void lfExtModifier::ModifyCoord_Dist_Poly5 (void *data, float *iocoord, int coun
     {
         float x = iocoord [0];
         float y = iocoord [1];
-        double ru = sqrt (x * x + y * y);
-        if (ru == 0.0)
+        double rd = sqrt (x * x + y * y);
+        if (rd == 0.0)
             continue;
 
         // Use Newton's method
-        double rd = ru;
+        double ru = rd;
         for (int step = 0; ; step++)
         {
-            double rd2 = rd * rd;
-            double frd = rd * (1.0 + k1 * rd2 + k2 * rd2 * rd2) - ru;
-            if (frd >= -NEWTON_EPS && frd < NEWTON_EPS)
+            double ru2 = ru * ru;
+            double fru = ru * (1.0 + k1 * ru2 + k2 * ru2 * ru2) - rd;
+            if (fru >= -NEWTON_EPS && fru < NEWTON_EPS)
                 break;
             if (step > 5)
                 // Does not converge, no real solution in this area?
                 goto next_pixel;
 
-            rd -= frd / (1.0 + 3 * k1 * rd2 + 5 * k2 * rd2 * rd2);
+            ru -= fru / (1.0 + 3 * k1 * ru2 + 5 * k2 * ru2 * ru2);
         }
-        if (rd < 0.0)
+        if (ru < 0.0)
             continue; // Negative radius does not make sense at all
 
-        rd /= ru;
-        iocoord [0] = x * rd;
-        iocoord [1] = y * rd;
+        ru /= rd;
+        iocoord [0] = x * ru;
+        iocoord [1] = y * ru;
 
     next_pixel:
         ;
     }
 }
 
-void lfExtModifier::ModifyCoord_UnDist_Poly5 (void *data, float *iocoord, int count)
+void lfExtModifier::ModifyCoord_Dist_Poly5 (void *data, float *iocoord, int count)
 {
     float *param = (float *)data;
-    // Ru = Rd * (1 + k1 * Rd^2 + k2 * Rd^4)
+    // Rd = Ru * (1 + k1 * Ru^2 + k2 * Ru^4)
     const float k1 = param [0];
     const float k2 = param [1];
 
@@ -575,50 +563,15 @@ void lfExtModifier::ModifyCoord_UnDist_Poly5 (void *data, float *iocoord, int co
     {
         const float x = iocoord [0];
         const float y = iocoord [1];
-        const float r2 = x * x + y * y;
-        const float poly4 = (1.0 + k1 * r2 + k2 * r2 * r2);
+        const float ru2 = x * x + y * y;
+        const float poly4 = (1.0 + k1 * ru2 + k2 * ru2 * ru2);
 
         iocoord [0] = x * poly4;
         iocoord [1] = y * poly4;
     }
 }
 
-void lfExtModifier::ModifyCoord_Dist_FOV1 (void *data, float *iocoord, int count)
-{
-    float *param = (float *)data;
-    // Rd = (1 / omega) * arctan (2 * Ru * tan (omega / 2))
-    float c1 = param [0];
-    float c2 = param [1];
-
-    for (float *end = iocoord + count * 2; iocoord < end; iocoord += 2)
-    {
-        const float x = iocoord [0];
-        const float y = iocoord [1];
-
-        iocoord [0] = c1 * atan (x * c2);
-        iocoord [1] = c1 * atan (y * c2);
-    }
-}
-
-void lfExtModifier::ModifyCoord_UnDist_FOV1 (void *data, float *iocoord, int count)
-{
-    float *param = (float *)data;
-    // Ru = tg (Rd * omega) / (2 * tg (omega/2))
-    float omega = param [0];
-    float dconst = param [1];
-
-    if (omega > 0.001)
-        for (float *end = iocoord + count * 2; iocoord < end; iocoord += 2)
-        {
-            const float x = iocoord [0];
-            const float y = iocoord [1];
-
-            iocoord [0] = tan (x * omega) * dconst;
-            iocoord [1] = tan (y * omega) * dconst;
-        }
-}
-
-void lfExtModifier::ModifyCoord_Dist_PTLens (void *data, float *iocoord, int count)
+void lfExtModifier::ModifyCoord_UnDist_PTLens (void *data, float *iocoord, int count)
 {
     float *param = (float *)data;
     float a = param [0];
@@ -630,39 +583,39 @@ void lfExtModifier::ModifyCoord_Dist_PTLens (void *data, float *iocoord, int cou
     {
         float x = iocoord [0];
         float y = iocoord [1];
-        double ru = sqrt (x * x + y * y);
-        if (ru == 0.0)
+        double rd = sqrt (x * x + y * y);
+        if (rd == 0.0)
             continue;
 
         // Use Newton's method
-        double rd = ru;
+        double ru = rd;
         for (int step = 0; ; step++)
         {
-            double frd = rd * (a * rd * rd * rd + b * rd * rd + c * rd + d) - ru;
-            if (frd >= -NEWTON_EPS && frd < NEWTON_EPS)
+            double fru = ru * (a * ru * ru * ru + b * ru * ru + c * ru + d) - rd;
+            if (fru >= -NEWTON_EPS && fru < NEWTON_EPS)
                 break;
             if (step > 5)
                 // Does not converge, no real solution in this area?
                 goto next_pixel;
 
-            rd -= frd / (4 * a * rd * rd * rd + 3 * b * rd * rd + 2 * c * rd + d);
+            ru -= fru / (4 * a * ru * ru * ru + 3 * b * ru * ru + 2 * c * ru + d);
         }
-        if (rd < 0.0)
+        if (ru < 0.0)
             continue; // Negative radius does not make sense at all
 
-        rd /= ru;
-        iocoord [0] = x * rd;
-        iocoord [1] = y * rd;
+        ru /= rd;
+        iocoord [0] = x * ru;
+        iocoord [1] = y * ru;
 
     next_pixel:
         ;
     }
 }
 
-void lfExtModifier::ModifyCoord_UnDist_PTLens (void *data, float *iocoord, int count)
+void lfExtModifier::ModifyCoord_Dist_PTLens (void *data, float *iocoord, int count)
 {
     float *param = (float *)data;
-    // Ru = Rd * (a * Rd^3 + b * Rd^2 + c * Rd + d)
+    // Rd = Ru * (a * Ru^3 + b * Ru^2 + c * Ru + d)
     const float a = param [0];
     const float b = param [1];
     const float c = param [2];
@@ -672,9 +625,9 @@ void lfExtModifier::ModifyCoord_UnDist_PTLens (void *data, float *iocoord, int c
     {
         const float x = iocoord [0];
         const float y = iocoord [1];
-        const float r2 = x * x + y * y;
-        const float r = sqrtf (r2);
-        const float poly3 = a * r2 * r + b * r2 + c * r + d;
+        const float ru2 = x * x + y * y;
+        const float r = sqrtf (ru2);
+        const float poly3 = a * ru2 * r + b * ru2 + c * r + d;
 
         iocoord [0] = x * poly3;
         iocoord [1] = y * poly3;
